///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPCUST.H
//
//  Description
//      Interface declaraction file for CXPLogon, CXPProvider and CMAPIStatus.
//      Property tag definitions.
//      Constant definitions.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.6
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
#ifndef _XPCUST_H
#define _XPCUST_H

#define STRICT
#include <WINDOWS.H>
#include <MAPIWIN.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIVAL.H>
#include <IMESSAGE.H>
#include <TNEF.H>

#ifdef _DEBUG
#define ENABLE_DEBUG_OUTPUT     1
#endif // _DEBUG

// Header file for the trace and logging functions.
#include "TRACES.H"

// Config Page Dialog Control IDs Message String ID's
#include "XPRC.H"

#define BASE_PROVIDER_ID        0x6600  // Look at the comments in MAPITAGS.H
#define PR_SKEL_ATTACH_FILE     PROP_TAG (PT_TSTRING, (BASE_PROVIDER_ID + 0x0001))
#define PR_SKEL_USER_NAME       PROP_TAG (PT_TSTRING, (BASE_PROVIDER_ID + 0x0002))


/// Names returned in the status object
#define TRANSPORT_DISPLAY_NAME_STRING       TEXT("PSS Skeletal Custom Processing TNEF Transport")
#define TRANSPORT_DLL_NAME_STRING           TEXT("XPCUST.DLL")

// Internal provider version
#define TRANSPORT_VERSION_MAJOR             1
#define TRANSPORT_VERSION_MINOR             1

#define MAX_NUM_OPTIONS                     2

// Name of the stream where the message properties get encoded using TNEF encapsulation
#define XP_TNEF_STREAM                      TEXT("WINMAIL.DAT")

#define MSG_FILENAME_PREFIX                 TEXT("XPCUST")
#define MSG_FILENAME_SUFFIX                 TEXT("DAT")
#define MSG_FILENAME_FORMAT                 TEXT("%s%x.%s")

#define ATT_FILENAME_PREFIX                 TEXT("XPATT")
#define ATT_FILENAME_SUFFIX 			    TEXT("DAT")
#define ATT_FILENAME_FORMAT                 TEXT("%s%x.%s")

#define W_ATT_FILENAME_PREFIX               OLESTR("XPATT")
#define W_ATT_FILENAME_SUFFIX               OLESTR("DAT")
#define W_ATT_FILENAME_FORMAT               OLESTR("%s%x.%s")

// Name of the attachment item within the storage object
#define ATTACH_ITEM_NAME            OLESTR("ATTACHMENT %d")

#define XP_CREATE_FLAGS         STGM_CREATE | \
                                STGM_DIRECT | \
                                STGM_READWRITE | \
                                STGM_SHARE_EXCLUSIVE
#define XP_OPEN_FLAGS           STGM_DIRECT | \
                                STGM_READWRITE | \
                                STGM_SHARE_EXCLUSIVE

// The order for this enumarations must match the order of the properties
// in the tag arrays. These arrays are used to access the 
// LPSPropValue arrays returned from MAPI interfaces. The use of enumerations
// hides differences when the number of properties in the tag array increases
// or decreases. It also helps maintain error-free indexing in the use of 
// the prop arrays
enum
{
    DISPLAY_NAME,           // Array Indexes
    EMAIL_ADDRESS,
    ADDRESS_TYPE,
    NUM_LOGON_PROPERTIES    // Array size
};

// ID array of property tags for the trannsport
const static SizedSPropTagArray(NUM_LOGON_PROPERTIES, sptLogonArray) =
{
    NUM_LOGON_PROPERTIES,
    {
        PR_SKEL_USER_NAME,
        PR_EMAIL_ADDRESS,
        PR_ADDRTYPE
    }
};

// Indexes of the identity property array
enum
{
    XPID_EID,               // Array Indexes
    XPID_NAME,
    XPID_SEARCH_KEY,
    NUM_SENDER_PROPS        // Array size
};

#define OUT_MSG_PROPS 4
static const SizedSPropTagArray(OUT_MSG_PROPS, sptOutMsgProps) =
{
    OUT_MSG_PROPS,
    {
        PR_SENDER_ENTRYID,
        PR_SENT_REPRESENTING_NAME,
        PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED,
        PR_HASATTACH
    }
};

// Used to generate one-off's for the sender of newly received messages
enum
{
    NEW_SENDER_NAME,
    NEW_SENDER_EMAIL,
    NEW_SENT_NAME,
    NEW_SENT_ADRTYPE,
    NEW_SENT_EMAIL,
    NEW_MSG_FLAGS,
    NEW_MSG_PROPS
};
static const SizedSPropTagArray(NEW_MSG_PROPS, sptNewMsgProps) =
{
    NEW_MSG_PROPS,
    {
        PR_SENDER_NAME,
        PR_SENDER_EMAIL_ADDRESS,

        PR_SENT_REPRESENTING_NAME,
        PR_SENT_REPRESENTING_ADDRTYPE,
        PR_SENT_REPRESENTING_EMAIL_ADDRESS,

        PR_MESSAGE_FLAGS
    }
};

// Used to generate transport-computed properties on received mail messages
#define MSG_RECIP_PROPS     2
static const SizedSPropTagArray(MSG_RECIP_PROPS, sptMsgRecipProps) =
{
    MSG_RECIP_PROPS,
    {
        PR_EMAIL_ADDRESS,
        PR_RECIPIENT_TYPE
    }
};
// Indexes to order the recipient's table of messages being submitted
enum
{
    RECIP_ROWID,
    RECIP_NAME,
    RECIP_EMAIL_ADR,
    RECIP_TYPE,
    RECIP_RESPONSIBILITY,
    RECIP_DELIVER_TIME,
    RECIP_REPORT_TEXT,
    RECIP_REPORT_TIME,
    TABLE_RECIP_PROPS
};
const static SizedSPropTagArray (TABLE_RECIP_PROPS, sptRecipTable) =
{
    TABLE_RECIP_PROPS,
    {
        PR_ROWID,
        PR_DISPLAY_NAME,
        PR_EMAIL_ADDRESS,
        PR_RECIPIENT_TYPE,
        PR_RESPONSIBILITY,
        PR_DELIVER_TIME,
        PR_REPORT_TEXT,
        PR_REPORT_TIME
    }
};

// Available properties in the status object of this provider
#define NUM_STATUS_OBJECT_PROPS    13
const static SizedSPropTagArray (NUM_STATUS_OBJECT_PROPS, sptStatusObj) =
{
    NUM_STATUS_OBJECT_PROPS,
    {
        PR_OBJECT_TYPE,
        PR_STATUS_CODE,
        PR_PROVIDER_DISPLAY,
        PR_PROVIDER_DLL_NAME,
        PR_RESOURCE_METHODS,
        PR_RESOURCE_FLAGS,
        PR_RESOURCE_TYPE,
        PR_STATUS_STRING,
        PR_DISPLAY_NAME,
        PR_IDENTITY_DISPLAY,
        PR_IDENTITY_ENTRYID,
        PR_IDENTITY_SEARCH_KEY,
        PR_CREATION_VERSION
    }
};

// Structure passed into the configuration dialog routine with
// the necessary information to fill the property sheet with
// the current values stored in the profile section of this provider.
typedef struct _XPDLG
{
    HINSTANCE      hInst;      // Instance of provider's DLL for loading resources for the UI
    HWND           hWnd;       // Handle of parent window or NULL is not UI should be used
    LPSPropValue * ppProps;    // Property array with the configuration properties
    LPMALLOC       pMallocObj; // IMalloc object for creating the MAPI objects and tables
    LPMAPISUP      pSupObj;    // IMAPISupport object for creating the UI property sheets
    ULONG          ulFlags;    // 0 or UI_READONLY for access control to the configuration values
    HANDLE         hUIMutex;   // Handle to the MUTEX to control exclusive access to the configuration UI
} XPDLG, *PXPDLG;

#define CONFIG_UI_MUTEX     TEXT("XPCUST_SERVICE_CONFIG_UI_MUTEX")

// Stand-alone helper functions (C style)
extern "C"
{
    int WINAPI PrivateMessageBox
                    (UINT                       ids, 
                     HWND                       hOwnerWnd,
                     UINT                       uFlags,
                     HINSTANCE                  hInst);
    HRESULT WINAPI DoLogonDlg
                    (PXPDLG                     pXPDlg);
    HRESULT WINAPI OpenProviderProfileSection
                    (LPPROVIDERADMIN            pAdminObj,
                     LPPROFSECT *               ppProfSect);
    MSGSERVICEENTRY ServiceEntry;
    HRESULT WINAPI GetMAPIError
                    (LPMAPIERROR *              ppMAPIError,
                     ULONG                      ulFlags,
                     HRESULT                    hError,
                     HINSTANCE                  hInstance);
    void WINAPI MergeConfigProps
                    (ULONG                      ulCfgProps,
                     LPSPropValue               pCfgProps,
                     LPSPropValue               pLogonProps);
    extern LPALLOCATEBUFFER    gpfnAllocateBuffer;  // MAPIAllocateBuffer function
    extern LPALLOCATEMORE      gpfnAllocateMore;    // MAPIAllocateMore function    
    extern LPFREEBUFFER        gpfnFreeBuffer;      // MAPIFreeBuffer function      
}

// Forward class declarations
class CXPProvider;
class CMAPIStatus;

// This next line just defines a token which gets expanded to nothing
#define IMPL

class CXPLogon : public IXPLogon
{
friend CMAPIStatus;
friend CXPProvider;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};
    MAPI_IXPLOGON_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this transport
//
private :
    STDMETHODIMP InitializePropArray
                    ();
    STDMETHODIMP GetAttachStream
                    (LPSTORAGE                  pRootObj,
                     LPATTACH                   pAttachObj,
                     ULONG                      ulCounter);
    STDMETHODIMP EncodeAttachments
                    (LPITNEF                    pTNEF,
                     LPMESSAGE                  pMsgObj);
    STDMETHODIMP WriteAttachsToFile
                    (LPMESSAGE                  pMessageObj,
                     DWORD                      dwFileID);
    STDMETHODIMP ReadAttachsFromFile
                    (LPMESSAGE                  pMessageObj,
                     LPSTORAGE                  pRootObj);
    STDMETHODIMP MakeSearchKey
                    (LPVOID                     pParentMemBlock,
                     LPTSTR                     pszAddrType,
                     LPTSTR                     pszAddress,
                     ULONG *                    pcbSearchKey,
                     LPBYTE *                   ppSearchKey);
    STDMETHODIMP GrowAdrList
                    (LPADRLIST *                ppAdrList,
                     ULONG                      ulResizeBy,
                     ULONG &                    ulOldAndNewCount);
    STDMETHODIMP SendMailMessage
                    (LPMESSAGE                  pMsgObj,
                     LPSRowSet                  pRecipRows);
    BOOL WINAPI LoadStatusString
                    (LPTSTR                     pString,
                     UINT                       uStringSize);
    void WINAPI InitializeTransportStatusFlags
                    (ULONG                      ulFlags);
    void WINAPI SetSessionFlags
                    (ULONG *                    pulFlags);
    void WINAPI SetIncomingProps
                    (LPMESSAGE                  pMsgObj);
    void WINAPI SetOutgoingProps
                    (LPMESSAGE                  pMsgObj,
                     FILETIME                   ft,
                     BOOL &                     fNeedDeliveryReport,
                     BOOL &                     fHasAttachments);
    void WINAPI UpdateStatus
                    ();
    void WINAPI IsThereAMessageWaiting
                    (ULONG &                    cMsgCount);
    BOOL WINAPI GetMsgFileName
                    (LPTSTR                     pszFileName,
                     DWORD &                    dwFileID);
    void WINAPI CheckSpoolerYield
                    (BOOL                       fReset = FALSE);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CXPLogon        (CXPProvider *              pXPProviderObj,
                     LPMAPISUP                  pSupObj,
                     LPSPropValue               pProps,
                     HANDLE                     hUIMutex);
    ~CXPLogon       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    ULONG               m_cRef;
    CXPLogon *          m_pNextSession;
    HANDLE              m_hUIMutex;
    CXPProvider *       m_pXPObj;  
    CMAPIStatus *       m_pStatusObj;
    LPMAPISUP           m_pSupObj;
    LPTSTR              m_pszAdrType;
    LPSPropValue        m_pProps;
    LPSPropValue        m_pIdentityProps;
    ULONG               m_ulTransportStatus;
    HINSTANCE           m_hInstance;
    ULONG               m_cOptData;
    LPOPTIONDATA        m_pOptData;
    TCHAR               m_szFileFound[MAX_PATH];
    HANDLE              m_hSearch;
    BOOL                m_fPendingProcess;
    DWORD               m_dwNextID;
};

class CXPProvider : public IXPProvider
{
friend CXPLogon;
friend CMAPIStatus;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};
    MAPI_IXPPROVIDER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    STDMETHODIMP InitializeStatusRow
                    (CXPLogon *                 pSession,
                     ULONG                      ulFlags);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CXPProvider     (HINSTANCE                  hInst,
                     LPMALLOC                   pMallocObj);
    ~CXPProvider    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef;
    LPMALLOC            m_pMallocObj;
    CRITICAL_SECTION    m_csTransport;
    CXPLogon *          m_pXPSessionList;
    HINSTANCE           m_hInstance;
};

class CMAPIStatus : public IMAPIStatus
{
friend CXPLogon;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};

    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAPISTATUS_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this transport
//
    STDMETHODIMP InitializePropArray 
                    (LPSPropValue *             ppPropValue);
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMAPIStatus     (CXPLogon *                 pXPObject,
                     LPPROFSECT                 pProfSectObj);
    ~CMAPIStatus    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG           m_cRef;
    CXPLogon *      m_pLogon;
    LPPROFSECT      m_pProfSectObj;
};

#endif // _XPCUST_H

// End of file for XPCUST.H
